!--------------------------------------------------------------------
!  PsDtgSen.f90 971020
!
!  Purpose:    Calculate ordered generalized real Schur decomposition.
!           The problem is:
!
!           A*x = lambda*B*x
!
!           and the routines returns ortogonal Q and Z, and upper
!           triangular S (only quasi-triangular) and P such that
!
!           A = Q*S*Z'  and B = Q*P*Z',
!
!           where Z' is the transpose of Z.
!           The generalized eigenvalues are
!           lambda = alpha/beta.
!
!           Note that the names of the matrices differ from what
!           Golub and van Loan (1989) use:
!
!           A = Q*T*Z'  and B = Q*S*Z'.
!
!--------------------------------------------------------------------
!     Header subroutine to convert the dimensions of a which were
!     passed as doubles to integer. Some FORTRAN compilers will
!     not accept array indices that are passed in as doubles.

SUBROUTINE PsDtgSen( A,B,Q,Z,AlphaR,AlphaI,BetaR,N,cutoff,pii_sii,GtIt, &
                     infosort,infocalc )

!DEC$ ATTRIBUTES C, REFERENCE, DLLEXPORT, ALIAS : 'PSDTGSEN' :: PsDtgSen

USE NUMERICAL_LIBRARIES

IMPLICIT NONE      !must be before declarations
REAL(8) A,B,Q,Z,AlphaR,AlphaI,BetaR,N,cutoff,pii_sii,GtIt,infosort,infocalc
INTEGER(4) Ni,infosorti,infocalci

Ni  = IDNINT(N)     !rounding double prec to integer

CALL PsDtgSen2( A,B,Q,Z,AlphaR,AlphaI,BetaR,Ni,cutoff,pii_sii,GtIt, &
                infosorti,infocalci )

infosort = DBLE(infosorti)
infocalc = DBLE(infocalci)

END
!--------------------------------------------------------------------


! *******************************************************************

SUBROUTINE PsDtgSen2( A,B,Q,Z,AlphaR,AlphaI,BetaR,N,cutoff,pii_sii,GtIt, &
                      infosort,infocalc )


IMPLICIT NONE      !must be before declarations

! ************************ Declarations, allocations ****************


! Integer dimension from wrapper subroutine
INTEGER(4) N
! Matrices from GAUSS
INTEGER(4) infosort,infocalc
REAL(8) A(N,N),B(N,N),Q(N,N),Z(N,N),AlphaR(N),AlphaI(N),BetaR(N), &
        cutoff,pii_sii,GtIt


! declarations, using dynamic allocation
REAL(8) WORK(:),pl,pr,dif(2)
INTEGER(4) LWORK,M,IWORK(:),LIWORK
LOGICAL(4) select(:)

ALLOCATABLE WORK,IWORK,select


LWORK = 4*N + 16 + N*(N+1)
LIWORK = 1


ALLOCATE( WORK(LWORK),IWORK(LIWORK),select(N) )


! ************************ Generalized Schur form, LAPACK  **********

Call DGEGS( 'V','V',N,A,N,B,N,AlphaR,AlphaI,BetaR,Q,N,Z,N,WORK,LWORK,infocalc )

! *************************** select for reordering **************


IF     (GtIt > 0.1_8 .AND. pii_sii > 0.1_8) THEN   ! lambda=alpha/beta
  ! abs(lambda) > cutoff first
  select = SQRT(AlphaR*AlphaR + AlphaI*AlphaI)>(ABS(BetaR)*cutoff)
ELSE IF (GtIt > 0.1_8 .AND. pii_sii < 0.1_8) THEN
  ! 1/abs(lambda) > cutoff first
  select = ABS(BetaR)>(SQRT(AlphaR*AlphaR + AlphaI*AlphaI)*cutoff)
ELSE IF (GtIt < 0.1_8 .AND. pii_sii > 0.1_8) THEN
  ! abs(lambda) < cutoff first
  select = SQRT(AlphaR*AlphaR + AlphaI*AlphaI)<(ABS(BetaR)*cutoff)
ELSE IF (GtIt < 0.1_8 .AND. pii_sii < 0.1_8) THEN
  ! 1/abs(lambda) < cutoff first
  select = ABS(BetaR)<(SQRT(AlphaR*AlphaR + AlphaI*AlphaI)*cutoff)
END IF

! *************************** Reordering, Poromaa *******************

IF ((infocalc == 0) .AND. ANY(select)) THEN
  CALL DTGSEN( 0, .TRUE., .TRUE.,select,N,A,N,B,N,AlphaR,AlphaI,BetaR, &
               Q,N,Z,N,M,pl,pr,dif,WORK,LWORK,IWORK,LIWORK,infosort )
END IF

! ************************ Deallocations, end **********************

DEALLOCATE(WORK,IWORK,select)

END
! ******************************************************************

