/*--------------------------------------------------------------------------
**  PsGees.prc
**
**
**
**
**
**
**
**
**
**
**
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch)
----------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------
**  zGees
**
**  Purpose:   Calculate a sorted (complex) Schur decomposition
**             of the complex matrix A.
**             It generates Q and T in
**
**                  A = Q*T*QH,
**
**             where QH is the transpose of the complex conjugate of Q,
**             Q is unitary and T is upper triangular with the eigenvalues
**             along the diagonal.
**
**
**  Usage:     dlibrary -a PszGees;   (only once)
**
**             { T,Q,lambda } = zGees( A,cutoff,GtIt )
**
**  Input:     A            NxN complex matrix
**             cutoff       scalar, abs(eigenvalues) < cutoff sorted first/last
**             GtIt         scalar, if GtIt=1, the large eigenvalues are
**                          ordered first; else they are ordered last
**
**  Output:    T            NxN  matrix (see above)
**             Q             ""
**             lambda       Nx1 complex vector, eigenvalues
**
**
**  Note:      This procedure calls on a dynamic library (dll file) created
**             by compiling and linking LAPACK 3.0. It uses the LAPACK driver
**             routine ZGEES.
**
**
**
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch), 20 March 1998
----------------------------------------------------------------------------*/

proc (3) = zGees( A,cutoff,GtIt );

  local N,Ar,Ai,Qr,Qi,lambdaR,lambdaI,T,Q,lambda,info,BadData;

  BadData = ismiss(A) + (not (abs(A)<1E+100));
  if  BadData > 0.1;
    retp( error(0),error(0),error(0) );
  endif;

  N = rows(A);

  Ar = A';
  Ai = zeros(N,N);
  Qr = zeros(N,N);                   /*GAUSS->FORTRAN*/
  Qi = Qr;
  lambdaR = zeros(1,N);
  lambdaI = lambdaR;
  info = -999.99;

  dllcall PSZGEES(Ar,Ai,N,Qr,Qi,lambdaR,lambdaI,cutoff,GtIt,info);
  T = complex(Ar',Ai');            /*FORTRAN -> GAUSS*/
  Q = complex(Qr',Qi');
  lambda = complex( lambdaR',lambdaI' );

  if round(info) /= 0;   /*information about convergence:*/
    T      = error(0);   /*0: successful; <0 illegal argument; >0 error*/
    Q      = error(0);   /*in computation (see zgees.f in Lapack for*/
    lambda = error(0);   /*more details)*/
  endif;

  retp( T,Q,lambda );

endp;
/*------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------
**  dGees
**
**  Purpose:   Calculate a sorted real Schur decomposition
**             of the real matrix A.
**             It generates Q and T in
**
**                    A = Q*T*Q',
**
**             where Q is orthogonal and T is upper quasi-triangular.
**
**
**  Usage:     dlibrary -a PsdGees;   (only once)
**
**             { T,Q,lambda } = dGees( A,cutoff,GtIt )
**
**  Input:     A            NxN real matrix
**             cutoff       scalar, abs(eigenvalues) < cutoff sorted first/last
**             GtIt         scalar, if GtIt=1, the large eigenvalues are
**                          ordered first; else they are ordered last
**
**  Output:    T            NxN  matrix (see above)
**             Q             ""
**             lambda       Nx1 complex vector, eigenvalues
**
**
**
**  Note:      This procedure calls on a dynamic library (dll file) created
**             by compiling and linking LAPACK 3.0. It uses the LAPACK driver
**             routine DGEES.
**
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch), 6 May 1997
----------------------------------------------------------------------------*/

proc (3) = dGees( A,cutoff,GtIt );

  local N,Q,lambdaR,lambdaI,T,lambda,info,BadData;

  BadData = ismiss(A) + (not (abs(A)<1E+100));
  if  BadData > 0.1;
    retp( error(0),error(0),error(0) );
  endif;

  N = rows(A);

  A = A';
  Q = zeros(N,N);                   /*GAUSS->FORTRAN*/
  lambdaR = zeros(1,N);
  lambdaI = lambdaR;
  info = -999.99;

  dllcall PSDGEES(A,N,Q,lambdaR,lambdaI,cutoff,GtIt,info);
  T = A';            /*FORTRAN -> GAUSS*/
  Q = Q';
  lambda = complex( lambdaR',lambdaI' );

  if round(info) /= 0;     /*information about convergence:*/
    T      = error(0);     /*0: successful; <0 illegal argument; >0 error*/
    Q      = error(0);     /*in computation (see dgees.f in Lapack for*/
    lambda = error(0);     /*more details)*/
  endif;

  retp( T,Q,lambda );

endp;
/*------------------------------------------------------------------------*/

