/*------------------------------------------------------------------------------
**  PsTgSen.prc
**
**
**
**
**
**
**
**
**
**
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch)
------------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------
**  zTgSen
**
**  Purpose:   Calculate a sorted generalized complex Schur decomposition
**             of A*x=lambda*B*x, where A and B are complex matrices.
**             It generates Q,Z,S, and P in
**
**               A = Q*S*ZH  (Golub and van Loan: A = Q*T*ZH)
**               B = Q*P*ZH  (Golub and van Loan: B = Q*S*ZH)
**
**             where ZH is the transpose of the complex conjugate of Z.
**             S and P are upper triangular complex matrices,
**             and Q and Z are unitary (i.e. QH = inv(Q) and ZH=inv(Z)).
**
**
**
**
**  Usage:     dlibrary -a PszTgSen;     (only once)
**
**             { S,P,Q,Z,lambda } =
**                                zTgSen( A,B,cutoff,pii_sii,GtIt ) or
**
**             in the notation of Golub and van Loan
**
**             { T,S,Q,Z,lambda } =
**                                zTgSen( A,B,cutoff,pii_sii,GtIt )
**
**  Input:     A            NxN real or complex matrix
**             B              ""
**             Cutoff       scalar, cutoff point. See below
**             pii_sii      scalar, governs the logical condition. See below
**             GtIt             ""
**
**  Output:    S            NxN complex matrix (see above)
**             P             ""
**             Q             ""
**             Z             ""
**             lambda       Nx1 complex vector with generalized eigenvalues
**                          computed as sii/pii (tii/sii acc to Golub and
**                          van Loan)
**
**
**  Sorting:   If sii are the diagonal elements of S and pii of P, then the
**             following blocks are put first:
**
**                      pii_sii = 1               pii_sii = 0
**        -----|----------------------------------------------------------
**    GtIt = 1 |  abs(sii)>abs(pii)*Cutoff    abs(pii)>abs(sii)*Cutoff
**             |
**    GtIt = 0 |  abs(sii)<abs(pii)*Cutoff    abs(pii)<abs(sii)*Cutoff
**
**
**             Since the generalized eigenvalues are lambda = sii/pii
**             (lambda = tii/sii acc to Golub and van Loan), this
**             table implies that the following blocks are put first:
**
**                      pii_sii = 1               pii_sii = 0
**        -----|----------------------------------------------------------
**    GtIt = 1 |  abs(lambda)>Cutoff          1/abs(lambda)>Cutoff
**             |
**    GtIt = 0 |  abs(lambda)<Cutoff          1/abs(lambda)<Cutoff
**
**
**
**
**  Note 1:    The naming conventions follow the LAPACK Users' Guide.
**             In contrast, Golub and van Loan (1996) use:
**             A = Q*T*ZH  and B = Q*S*ZH.
**
**  Note 2:    Setting B=I gives a standard eigenvalue problem.
**             In this case a standard (not generalized) Schur decomposition
**             can be useful if the eigenvectors are linearly dependent.
**
**  Note 3:    This procedure calls on a dynamic library (dll file) created
**             by compiling and linking my FORTRAN code PsZtgSen.F90 with the
**             FORTRAN library Lapack.lib created from LAPACK 3.0.
**             The calculations are made in complex(8) in FORTRAN 90.
**
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch), 20 March 1998,
**  documentation updated 9 Decemeber 1999
----------------------------------------------------------------------------*/

proc (5) = zTgSen( A,B,Cutoff,pii_sii,GtIt );

  local Ar,Br,N,Ai,Bi,Qrr,Qi,Zr,Zi,S,P,Q,Z,AlphaR,AlphaI,BetaR,BetaI,lambda,
        infosort,infocalc,BadData;

  BadData = ismiss(A|B) + (not (abs(A|B)<1E+100));
  if  BadData > 0.1;
    retp( error(0),error(0),error(0),error(0),error(0) );
  endif;


  Ar = real(A');   /*real part of A: easier to pass doubles than complex*16*/
  Br = real(B');   /*to FORTRAN*/
  Ai = imag(A');   /*imaginary part of A*/
  Bi = imag(B');

  N = rows(Ar);

  Qrr = zeros(N,N);   /*empty matrices to be filled by FORTRAN*/
  Qi  = zeros(N,N);
  Zr  = zeros(N,N);
  Zi  = zeros(N,N);

  AlphaR  = zeros(1,N);
  AlphaI  = AlphaR;
  BetaR   = AlphaR;
  BetaI   = AlphaR;
  infosort = -999.99;
  infocalc = infosort;

  dllcall PSZTGSEN(Ar,Ai,Br,Bi,Qrr,Qi,Zr,Zi,AlphaR,AlphaI,BetaR,BetaI,
                   N,Cutoff,pii_sii,GtIt,infosort,infocalc);

  S = complex( Ar',Ai' ); /*transpose since FORTRAN has column-major order*/
  P = complex( Br',Bi' ); /*converting to complex matrices again*/
  Q = complex( Qrr',Qi' );
  Z = complex( Zr',Zi' );
  lambda =  complex(AlphaR',AlphaI')./complex(BetaR',BetaI');

  if (round(infosort) /= 0) or  (round(infocalc) /= 0);
    S      = error(0);   /*infosort: scalar, information about convergence*/
    P      = error(0);   /*0: successful; <0 illegal argument; >0 error*/
    Q      = error(0);   /*in computation (see ztgsen.f in Lapack for*/
    Z      = error(0);   /*more details)*/
    lambda = error(0);   /*infocalc: as infosort, but for decomposition*/
  endif;

  retp( S,P,Q,Z,lambda );

endp;
/*------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------
**  dTgSen
**
**  Purpose:   Calculate a sorted generalized real Schur decomposition
**             of A*x=lambda*B*x, where A and B are real matrices.
**             It generates Q,Z,S, and P in
**
**               A = Q*S*Z'  (Golub and van Loan: A = Q*T*Z')
**               B = Q*P*Z'  (Golub and van Loan: B = Q*S*Z')
**
**             where Z' is the transpose of Z.
**             S and P are upper (quasi if S) triangular matrices,
**             and Q and Z are orthogonal (i.e. QH = inv(Q) and ZH=inv(Z)).
**
**
**
**
**  Usage:     dlibrary -a PsDtgSen;     (only once)
**
**             { S,P,Q,Z,lambda } =
**                                 dTgSen( A,B,cutoff,pii_sii,GtIt ) or
**
**             in the notation of Golub and van Loan
**
**             { T,S,Q,Z,lambda } =
**                                 dTgSen( A,B,cutoff,pii_sii,GtIt )
**
**
**  Input:     A            NxN real matrix
**             B              ""
**             Cutoff       scalar, cutoff point. See below.
**             pii_sii      scalar, governs the logical condition. See below.
**             GtIt             ""
**
**  Output:    S            NxN real matrix (see above)
**             P             ""
**             Q             ""
**             Z             ""
**             lambda       Nx1 complex vector with generalized eigenvalues.
**                          Note that these are not equal to sii/pii in the
**                          real generalized Schur decomnposition if there
**                          are repeated roots.
**
**
**
**
**  Sorting:   If the generalized eigenvalues are lambda=alpha/beta, then the
**             following blocks are put first:
**
**                      pii_sii = 1               pii_sii = 0
**        -----|----------------------------------------------------------
**    GtIt = 1 |  abs(alpha)>abs(beta)*Cutoff    abs(beta)>abs(alpha)*Cutoff
**             |
**    GtIt = 0 |  abs(alpha)<abs(beta)*Cutoff    abs(beta)<abs(alpha)*Cutoff
**
**
**             Since the generalized eigenvalues are lambda = alpha/beta, this
**             table implies that the following blocks are put first:
**
**                      pii_sii = 1               pii_sii = 0
**        -----|----------------------------------------------------------
**    GtIt = 1 |  abs(lambda)>Cutoff             1/abs(lambda)>Cutoff
**             |
**    GtIt = 0 |  abs(lambda)<Cutoff             1/abs(lambda)<Cutoff
**
**
**
**
**
**
**  Note 1:    The naming conventions follow the LAPACK Users' Guide.
**             In contrast, Golub and van Loan (1996) use:
**             A = Q*T*Z'  and B = Q*S*Z'.
**
**  Note 2:    Setting B=I gives a standard eigenvalue problem.
**             In this case a standard (not generalized) Schur decomposition
**             can be useful if the eigenvectors are linearly dependent.
**
**  Note 3:    This procedure calls on a dynamic library (dll file) created
**             by compiling and linking my FORTRAN code PsDtgSen.F90 with the
**             FORTRAN library Lapack.lib created from LAPACK 3.0.
**             The calculations are made in real(8) in FORTRAN 90.
**
**
**
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch), 20 March 1998,
**  documentation updated 9 Decemeber 1999
----------------------------------------------------------------------------*/

proc (5) = dTgSen( A,B,Cutoff,pii_sii,GtIt );

  local AlphaR,AlphaI,Beta,N,Q,Z,S,P,lambda,infosort,infocalc,Alpha,BadData;

  BadData = ismiss(A|B) + (not (abs(A|B)<1E+100));
  if  BadData > 0.1;
    retp( error(0),error(0),error(0),error(0),error(0) );
  endif;

  A = A';
  B = B';

  N = rows(A);

  Q  = zeros(N,N);   /*empty matrices to be filled by FORTRAN*/
  Z  = zeros(N,N);
  AlphaR = zeros(1,N);
  AlphaI = AlphaR;
  Beta   = AlphaR;
  infosort = -999.99;
  infocalc = infosort;

  dllcall PSDTGSEN(A,B,Q,Z,AlphaR,AlphaI,Beta,N,
                   Cutoff,pii_sii,GtIt,infosort,infocalc);

  S = A';    /*transpose since FORTRAN has column-major order*/
  P = B';
  Q = Q';
  Z = Z';
  Alpha = complex( AlphaR',AlphaI' );
  lambda = Alpha./(Beta');

  if (round(infosort) /= 0) or  (round(infocalc) /= 0);
    S      = error(0);   /*infosort: scalar, information about convergence*/
    P      = error(0);   /*0: successful; <0 illegal argument; >0 error*/
    Q      = error(0);   /*in computation (see dtgsen.f in Lapack for*/
    Z      = error(0);   /*more details)*/
    lambda = error(0);   /*infocalc: as infosort, but for decomposition*/
  endif;

  retp( S,P,Q,Z,lambda );

endp;
/*------------------------------------------------------------------------*/

