!--------------------------------------------------------------------
!  PsZtgSen.f90 970805
!
!  Purpose: Calculate ordered generalized Schur decomposition by
!         using LAPACK routines.
!
!           The problem is:
!
!           A*x = lambda*B*x
!
!           and the routines returns unitary Q and Z, and upper
!           triangular S and P such that
!
!           A = Q*S*Zh  and B = Q*P*Zh,
!
!           where Zh is the transpose of the complex conjuhate of Z.
!           The generalized eigenvalues are lambda = Sii/Pii or
!           lambda = alpha/beta.
!
!           Note that the names of the matrices differ from what
!           Golub and van Loan (1989) use:
!
!           A = Q*T*Zh  and B = Q*S*Zh, with lambda = Tii/Sii.
!
!
!--------------------------------------------------------------------
!     Header subroutine to convert the dimensions of a which were
!     passed as doubles to integer. Some FORTRAN compilers will
!     not accept array indices that are passed in as doubles.

SUBROUTINE PsZtgSen( APSr,APSi,BPSr,BPSi,QPSr,QPSi,ZPSr,ZPSi, &
                     AlphaR,AlphaI,BetaR,Betaii,N,cutoff,pii_sii,GtIt, &
                     infosort,infocalc )

!DEC$ ATTRIBUTES C, REFERENCE, DLLEXPORT, ALIAS : 'PSZTGSEN' :: PsZtgSen

USE NUMERICAL_LIBRARIES

IMPLICIT NONE      !must be before declarations
REAL(8) APSr,APSi,BPSr,BPSi,QPSr,QPSi,ZPSr,ZPSi,AlphaR,AlphaI,BetaR,Betaii,N, &
        cutoff,pii_sii,GtIt,infosort,infocalc
INTEGER(4) Ni,infosorti,infocalci

Ni  = IDNINT(N)     !rounding double prec to integer

CALL PsZtgSen2(APSr,APSi,BPSr,BPSi,QPSr,QPSi,ZPSr,ZPSi, &
               AlphaR,AlphaI,BetaR,Betaii,Ni, &
               cutoff,pii_sii,GtIt,infosorti,infocalci)

infosort = DBLE(infosorti)
infocalc = DBLE(infocalci)

END
!--------------------------------------------------------------------


! *******************************************************************

SUBROUTINE PsZtgSen2( APSr,APSi,BPSr,BPSi,QPSr,QPSi,ZPSr,ZPSi, &
                      AlphaR,AlphaI,BetaR,Betaii,N, &
                      cutoff,pii_sii,GtIt,infosort,infocalc )


IMPLICIT NONE      !must be before declarations

! ************************ Declarations, allocations ****************


! Integer dimension from wrapper subroutine
INTEGER(4) N
! Matrices from GAUSS
INTEGER(4) infosort,infocalc
REAL(8) APSr(N,N),APSi(N,N),BPSr(N,N),BPSi(N,N), &
        QPSr(N,N),QPSi(N,N),ZPSr(N,N),ZPSi(N,N), &
        AlphaR(N),AlphaI(N),BetaR(N),Betaii(N), &
        cutoff,pii_sii,GtIt


! declarations, using dynamic allocation
COMPLEX(8) A(:,:),B(:,:),WORK(:),ALPHA(:),BETA(:),Q(:,:),Z(:,:)

REAL(8) RWORK(:),pl,pr,dif(2)
INTEGER(4) LWORK,LRWORK,M,IWORK(:),LIWORK
LOGICAL(4) select(:)

ALLOCATABLE A,B,WORK,ALPHA,BETA,Q,Z,RWORK,IWORK,select


LWORK  = 3*N*N+N+8
LRWORK = 3*N
LIWORK = LWORK   !Unclear what the dimension of IWORK should be

ALLOCATE( A(N,N),B(N,N),WORK(LWORK),ALPHA(N),BETA(N),Q(N,N),Z(N,N), &
          RWORK(LRWORK),IWORK(LIWORK),select(N) )


A = CMPLX( APSr,APSi,8 )     !two double prec to one complex
B = CMPLX( BPSr,BPSi,8 )


! ************************ Generalized Schur form, LAPACK  **********

Call ZGEGS( 'V','V',N,A,N,B,N,ALPHA,BETA,Q,N,Z,N,WORK,LWORK,RWORK,infocalc )

! *************************** select for reordering **************


IF     (GtIt > 0.1_8 .AND. pii_sii > 0.1_8) THEN  ! lambda= alpha/beta
  select = ABS(ALPHA) > (ABS(BETA)*cutoff)  ! abs(lambda) > cutoff first
ELSE IF (GtIt > 0.1_8 .AND. pii_sii < 0.1_8) THEN
  select = ABS(BETA)  > (ABS(ALPHA)*cutoff) ! 1/abs(lambda) > cutoff first
ELSE IF (GtIt < 0.1_8 .AND. pii_sii > 0.1_8) THEN
  select = ABS(ALPHA) < (ABS(BETA)*cutoff)  ! abs(lambda) < cutoff first
ELSE IF (GtIt < 0.1_8 .AND. pii_sii < 0.1_8) THEN
  select = ABS(BETA)  < (ABS(ALPHA)*cutoff)  ! 1/abs(lambda) < cutoff first
END IF

! *************************** Reordering, Poromaa *******************

IF ((infocalc == 0) .AND. ANY(select)) THEN
  CALL ZTGSEN( 0, .TRUE., .TRUE.,select,N,A,N,B,N,ALPHA,BETA, &
               Q,N,Z,N,M,pl,pr,dif,WORK,LWORK,IWORK,LIWORK,infosort )
END IF
! ************************ Writing to GAUSS matrices ***************

APSr    = DBLE(A)
APSi    = DBLE( AIMAG(A) )
BPSr    = DBLE(B)
BPSi    = DBLE( AIMAG(B) )
QPSr    = DBLE(Q)
QPSi    = DBLE( AIMAG(Q) )
ZPSr    = DBLE(Z)
ZPSi    = DBLE( AIMAG(Z) )
AlphaR  = DBLE(ALPHA)
AlphaI  = DBLE( AIMAG(ALPHA) )
BetaR   = DBLE(BETA)
Betaii  = DBLE( AIMAG(BETA) )

! ************************ Deallocations, end **********************

DEALLOCATE(A,B,WORK,ALPHA,BETA,Q,Z,RWORK,IWORK,select)

END
! ******************************************************************

